#!/usr/local/bin/tclsh 
# 
#	  Name: sfa (Tcl Script) Send File ASCII 
#		to prove implementability of UFT in Tcl 
#	  Date: 1996-Jul-16 
# 
 
set verbose 0 
 
# 
# confirm sufficient arguments: 
if {$argc != 2} then { 
	puts stderr "Usage: sfa <file> <user>" 
	exit 24 
	} 
 
# 
# process file name: 
set NAME [lindex $argv 0] 
if {$NAME == "-"} then { 
	set NAME ""; set F stdin; set SIZE 0 
	} else { 
	set F [open $NAME RDONLY] 
	set SIZE [file size $NAME] 
	} 
 
# 
# process recipient name: 
set USER [lindex $argv 1] 
set HOST [lindex [split $USER "@"] 1] 
set USER [lindex [split $USER "@"] 0] 
if {$HOST == ""} then {set HOST "localhost"} 
 
# 
# try to connect to server: 
set S [socket $HOST 608] 
fconfigure $S -buffering line -translation {auto crlf} 
 
# 
# now a proc to "Wait for ACK": 
proc uftcwack { sock } { 
	global RESPONSE 
	while {1} { 
		set BC [gets $sock BS] 
		if {$BC < 0} {set RESPONSE "5XX EOF"; return 1} 
		if {$BC == 0} then {set RESPONSE "2XX ACK"; return 0} 
		set RESPONSE $BS 
		set TS [string range $BS 0 0] 
		if {$TS == "1"} then {continue} 
		if {$TS == "2"} then {return 0} 
		if {$TS == "3"} then {return 0} 
		if {$TS == "4"} then {return 1} 
		if {$TS == "5"} then {return 1} 
		if {$TS == "6"} then {continue} 
		return 1 
		} 
	} 
 
# 
# consume herald: 
if {$verbose} then {puts stderr "herald: [gets $S]"} 
 
# 
# build and send a FILE command: 
set FROM "-" 
if {[info exists env(USER)]} then {set FROM $env(USER)} 
if {[info exists env(LOGNAME)]} then {set FROM $env(LOGNAME)} 
set AUTH "-" 
puts $S "FILE $SIZE $FROM $AUTH" 
if {[uftcwack $S] != 0} then { 
	puts stderr $RESPONSE 
	exit -1 
	} 
if {$verbose} then {puts stderr "uftcwack: $RESPONSE"} 
 
# 
# this version of SENDFILE only does ASCII type files: 
puts $S "TYPE A" 
if {[uftcwack $S] != 0} then { 
	puts stderr $RESPONSE 
	exit -1 
	} 
if {$verbose} then {puts stderr "uftcwack: $RESPONSE"} 
 
# 
# identify the target user: 
puts $S "USER $USER" 
if {[uftcwack $S] != 0} then { 
	puts stderr $RESPONSE 
	exit -1 
	} 
if {$verbose} then {puts stderr "uftcwack: $RESPONSE"} 
 
# 
# identify the file (if it has a name): 
if {$NAME != ""} then { 
puts $S "NAME $NAME" 
if {[uftcwack $S] != 0} then { 
	puts stderr $RESPONSE 
	exit -1 
	} 
if {$verbose} then {puts stderr "uftcwack: $RESPONSE"} 
} 
 
while {1} { 
	set BC [gets $F BS] 
	if {$BC < 0} break 
	puts $S "DATA [expr $BC + 2]" 
	puts $S $BS 
	if {[uftcwack $S] != 0} then { 
		puts stderr $RESPONSE 
		exit -1 
		} 
	} 
 
puts $S "EOF" 
if {[uftcwack $S] != 0} then { 
	puts stderr $RESPONSE 
	exit -1 
	} 
if {$verbose} then {puts stderr "uftcwack: $RESPONSE"} 
 
puts $S "QUIT" 
if {[uftcwack $S] != 0} then { 
	puts stderr $RESPONSE 
	exit -1 
	} 
if {$verbose} then {puts stderr "uftcwack: $RESPONSE"} 
 
close $S 
close $F 
 
exit 
 

